function [fisher_train,fisher_test,fisher_basis]=fisher(data_train,data_test,no_classes,no_train,no_test);
%-------------- Define variables ---------------------------------
% c - 12 original training images
% mn - mean of c
% m(:,1) - mean of class 1
% m(:,2) - mean of class 2
% msc - mean (of each class) subtracted training images
% sw - within class scatter matrix
% sb - between class scatter matrix
% w - Fisher basis vectros
% ccc - original training images projected onto Fisher basis vectors
%------------------------------------------------------------------

% Define values
N=size(data_train,1);
P=size(data_train,2);
NumClasses = no_classes;
ImgsPerClass = no_train;
ImgsTestPerClass = no_test;
tol = .000001;

c=data_train;
c_test=data_test;

% compute mean of each class
mn = mean(c')';
for i = 1:NumClasses 
    m(:,i) = mean(c(:,(i-1)*ImgsPerClass+1:(i-1)*ImgsPerClass+ImgsPerClass)')';
    msm(:,i) = m(:,i) - mn;
end

% Mean subtract all the images
for i=1:NumClasses*ImgsPerClass
    msc(:,i) = c(:,i) - m(:,double(floor((i-1)/ImgsPerClass))+1);
end

% Calculate the sw matrix (within class scatter)
sw = zeros(N);
for i=1:NumClasses*ImgsPerClass
    sw = sw + msc(:,i) * msc(:,i)';
end

% Calculate the sb matrix (between class scatter)
sb = zeros(N);
for i=1:NumClasses
    sb = sb + msm(:,i) * msm(:,i)';
end

% Solved the generalized eigenvalue problem sb*w = d*sw*w
[V,D]=eig(sb,sw);
szd = size(D);
for i=1:szd(1)
    evals(i) = D(i,i);
end
[a,b]=sort(evals);

% Pull off the eigenvectors associated with the  c-1 most significant eigenvalues
for i = 1:NumClasses-1
    w(:,i)=V(:,b(szd(1)-(i-1)));
end

% Check that eig worked
norm((sb*w)-(D(b(szd(1)),b(szd(1)))*sw*w));

% project training data classes onto eigenvectors
for i=1:NumClasses*ImgsPerClass
    fisher_train(:,i) =  real(w' * c(:,i));
end

% project test data classes onto eigenvectors
for i=1:NumClasses*ImgsTestPerClass
    fisher_test(:,i) =  real(w' * c_test(:,i));
end

fisher_basis=w;